int open_socket_to_send_data(int tcp_port_num);


/*
 * readn - this function reads "n" bytes from a socket before returning.
 */
int readn(register int fd, register char *ptr, register int nbytes);

/*
 * read_reply_from_socket - read reply from server
 */
int read_reply_from_socket(int sock,struct reply_struct *the_reply);
     


/*
 * writen - this function writes "n" bytes down a socket before returning.
 */
int writen(register int fd, register char *ptr, register int nbytes);

/* 
 * write_request_to_socket - sends the reply to the client who requested
 *                           service
 */
int write_request_to_socket(int sock, struct request_struct *the_request);

/* 
 * ipc_comm - sends a request to the server and gets a reply from it
 */
int ipc_comm(struct request_struct *the_request, struct reply_struct *the_reply);

/* 
 * process_???_reply - processes reply from the server to extract the
 *                     appropriate information 
 */
int process_state_reply(struct reply_struct *the_reply);

int process_infrared_reply(struct reply_struct *the_reply);

int process_sonar_reply(struct reply_struct *the_reply);

int process_configuration_reply(struct reply_struct *the_reply);

int process_conf_reply(struct reply_struct *the_reply, long *conf);

int process_velocity_reply(struct reply_struct *the_reply);

int process_acceleration_reply(struct reply_struct *the_reply);

int process_compass_reply(struct reply_struct *the_reply);

int process_bumper_reply(struct reply_struct *the_reply);

int process_laser_reply(struct reply_struct *the_reply);

int process_predict_reply(struct reply_struct *the_reply, long *state,
			  int *laser);

int process_simple_reply(struct reply_struct *the_reply);

int process_obstacle_reply(struct reply_struct *the_reply, long *obs);

int process_rpx_reply(struct reply_struct *the_reply, long *robot_pos);

int process_send_reply(struct reply_struct *the_reply);

/* 
 * process_message_reply 
 */
process_message_reply(struct reply_struct *the_reply);

int process_socket_reply(struct reply_struct *the_reply);

int process_special_reply(struct reply_struct *the_reply, char *data);

/*****************************
 *                           *
 * Robot Interface Functions *
 *                           *
 *****************************/

/*
 * create_robot - requests the server to create a robot with
 *                id = robot_id and establishes connection with
 *                the robot. This function is disabled in this
 *                version of the software.
 * 
 * parameters:
 *    long robot_id -- id of the robot to be created. The robot
 *                     will be referred to by this id. If a process
 *                     wants to talk (connect) to a robot, it must
 *                     know its id.
 */
int create_robot(long robot_id);


/*
 * connect_robot - requests the server to connect to the robot
 *                 with id = robot_id. In order to talk to the server,
 *                 the SERVER_MACHINE_NAME and SERV_TCP_PORT must be
 *                 set properly. If a robot with robot_id exists,
 *                 a connection is established with that robot. If
 *                 no robot exists with robot_id, no connection is
 *                 established. Currently, you should call connect_robot(1)
 *                 since the pre-existed (when server was started) robot's 
 *                 id is 1.
 *
 * parameters:
 *    long robot_id -- robot's id. In order to connect to a robot, you
 *                     must know it's id.
 */
int connect_robot(long robot_id);

/*
 * disconnect_robot - requests the server to close the connect with robot
 *                    with id = robot_id. 
 *
 * parameters:
 *    long robot_id -- robot's id. In order to disconnect a robot, you
 *                     must know it's id.
 */
int disconnect_robot(long robot_id);

/* 
 * ac - sets accelerations of the robot. Currently it has no effect in 
 *      simulation mode.
 *
 * parameters:
 *    int t_ac, s_ac, r_ac -- the translation, steering, and turret
 *                            accelerations. t_ac is in 1/10 inch/sec^2
 *                            s_ac and r_ac are in 1/10 degree/sec^2.
 */
int ac(int t_ac, int s_ac, int r_ac);

/*
 * sp - sets speeds of the robot, this function will not cause the robot to
 *      move. However, the set speed will be used when executing a pr()
 *      or a pa().
 *
 * parameters:
 *    int t_sp, s_sp, r_sp -- the translation, steering, and turret
 *                            speeds. t_sp is in 1/10 inch/sec and
 *                            s_sp and r_sp are in 1/10 degree/sec.
 */
int sp(int t_sp, int s_sp, int r_sp);

/*
 * gs - get the current state of the robot according to the mask (of 
 *      the communication channel)
 */
int gs(void);

/*
 * pr - moves the motors of the robot by a relative distance, using the speed
 *      set by sp(). The three parameters specify the relative distances for
 *      the three motors: translation, steering, and turret. All the three
 *      motors move concurrently if the speeds are not set to zero and the 
 *      distances to be travelled are non-zero. Depends on the timeout 
 *      period set (by function conf_tm (timeout), the motion may 
 *      terminate before the robot has moved the desired distance
 *
 * parameters:
 *    long t_pr, s_pr, r_pr -- the specified relative positions of the
 *                             translation, steering, and turret motors.
 *                             t_pr is in 1/10 inch and s_pr and r_pr are
 *                             in 1/10 degrees.
 */
int pr(long t_pr, long s_pr, long r_pr);

/*
 * pa - moves the motors of the robot to the specified absolute positions 
 *      using the speed set by sp(). Depends on the timeout period set 
 *      (by conf_tm ()), the motion may terminate before the robot has 
 *      moved the desired distance.
 *
 * parameters:
 *    long t_pa, s_pa, r_pa -- the specified absolute positions of the
 *                             translation, steering, and turret motors.
 *                             t_pa is in 1/10 inch and s_pa and r_pa are
 *                             in 1/10 degrees.
 */
int pa(long t_pa, long s_pa, long r_pa);

/*
 * ws - waits for stop of motors of the robot. This functions is intended  
 *      to be used in conjunction with pr() and pa() to detect the desired
 *      motion has finished
 *
 * parameters:
 *    unsigned char w_t, w_s, w_r --  These three parameters specify 
 *                                    which axis or combination of axis 
 *                                    (translation, steering, and turret) 
 *                                    to wait. 
 *    unsigned char timeout -- specifies how long to wait before timing out 
 *                             (return without stopping the robot).
 */
int ws(unsigned char w_t, unsigned char w_s, unsigned char w_r,
       unsigned char timeout);

/*
 * vm - velocity mode, command the robot to move at translational
 *      velocity = tv, steering velocity = sv, and rotational velocity =
 *      rv. The robot will continue to move at these velocities until
 *      either it receives another command or this command has been
 *      timeout (in which case it will stop its motion).
 *
 * parameters: 
 *    int tv, sv, rv -- the desired translation, steering, and turret
 *                      velocities. tv is in 1/10 inch/sec and
 *                      sv and rv are in 1/10 degree/sec.
 */
int vm(int tv, int sv, int rv);

/*
 * st - stops the robot (the robot holds its current position)
 */
int st(void);

/*
 * lp - set motor limp (the robot may not hold its position).
 */
int lp(void);

/*
 * bits_to_byte - converts 8 bits into one byte. Helper function for ct()
 */
unsigned char bits_to_byte(char bt0, char bt1, char bt2, char bt3, char bt4,
			   char bt5, char bt6, char bt7);

/*
 * ct - send the sensor mask, Smask, to the robot. You must first change
 *      the global variable Smask to the desired communication mask before
 *      calling this function. 
 */
int ct (void);

/*
 * tk - sends the character stream, strm, to the voice synthesizer
 *      to make the robot talk.
 *
 * parameters:
 *    char *strm -- the string to be sent to the synthesizer.
 *
 * for example tk("Hello, how are you.")
 */
int tk(char *strm);

/*
 * zr - zeroing the robot, align steering and turret with bumper zero.
 *      The position, steering and turret angles are all set to zero.
 *      This function returns when the zeroing process has completed.
 */
int zr(void);

/*
 * dp - define the current position of the robot as (x,y)
 * 
 * parameters:
 *    long x, y -- the position to set the robot to.
 */
int dp(long x, long y);


/*
 * da - define the current steering angle of the robot to be th
 *      and the current turret angle of the robot to be tu.
 * 
 * parameters:
 *    int th, tu -- the steering and turret orientations to set the
 *                  robot to.
 */
int da(int th, int tu);

/*
 * conf_ir - configure infrared sensor system.
 *
 * parameters: 
 *    int history -- specifies the percentage dependency of the current 
 *                   returned reading on the previous returned reading.
 *                   It should be set between 0 and 10: 0 = no dependency 
 *                   10 = full dependency, i.e. the reading will not change
 *    int order[16] --  specifies the firing sequence of the infrared 
 *                      "255". For example, if you want to use only the 
 *                      front three infrared sensors then set order[0]=0,
 *                      order[1]=1, order[2]=15, order[3]=255 (terminator).
 */
int conf_ir (int history, int order[16]);

/*
 * conf_sn - configure sonar sensor system.
 *
 * parameters:
 *    int rate -- specifies the firing rate of the sonar in 4 milli-seconds 
 *                interval; 
 *                     You can terminate the order list by a "255". For 
 *                     example, if you want to use only the front three 
 *                     sensors, then set order[0]=0, order[1]=1, order[2]=15, 
 *                     order[3]=255 (terminator).
 */
int conf_sn (int rate, int order[16]);

/*
 * conf_cp - configure compass system.
 * 
 * parameters:
 *    int mode -- specifies compass on/off: 0 = off ; 1 = on; 2 = calibrate.
 */
int conf_cp(int mode);

/*
 * conf_ls - configure laser sensor system:
 *
 * parameters:
 *    unsigned int mode -- specifies the on-board processing mode of the laser 
 *                         sensor data which determines the mode of the data 
 *                         coming back: 
 *                           the first bit specifies the on/off;
 *                           the second bit specifies point/line mode;
 *                           the third to fifth bits specify the 
 *                           returned data types: 
 *                             000 = peak pixel, 
 *                             001 = rise pixel, 
 *                             010 = fall pixel, 
 *                             011 = magnitude,
 *                             100 = distance;
 *                           the sixth bit specifies data integrity checking.
 *
 *   unsigned int threshold -- specifies the inverted acceptable brightness
 *                             of the laser line. 
 *
 *   unsigned int width -- specifies the acceptable width in terms
 *                         of number of pixels that are brighter than the 
 *                         set threshold.
 *  
 *   unsigned int num_data -- specifies the number of sampling points. 
 *   unsigned int processing --  specifies the number of neighboring 
 *                               pixels for averaging
 *
 * If you don't understand the above, try this one:
 *   conf_ls(51, 20, 20, 20, 4)
 */
int conf_ls(unsigned int mode, unsigned int threshold, unsigned int width,
	    unsigned int num_data, unsigned int processing);

/*
 * conf_sg - configure laser sensor system line segment processing mode:
 *
 * parameters:
 *    unsigned int threshold -- specifies the threshold value for least-square
 *                             fitting. When the error term grows above the 
 *                             threshold, the line segment will be broken
 *    unsigned int min_points -- specifies the acceptable number of points
 *                              to form a line segment.
 *    unsigned int gap -- specifies the acceptable "gap" between two segments
 *                        while they can still be treated as one 
 *                        (in  1/10 inch)
 *
 * If you don't understand the above, try this one:
 *    conf_sg(50, 4, 30)
 */
int conf_sg(unsigned int threshold, unsigned int min_points, unsigned int gap);

/*
 * conf_tm - sets the timeout period of the robot in seconds. If the
 *           robot has not received a command from the host computer
 *           for more than the timeout period, it will abort its 
 *           current motion
 * 
 * parameters:
 *    int timeout -- timeout period in seconds. If it is 0, there will 
 *                   be no timeout on the robot.
 */
int conf_tm(int timeout);

/*
 * get_ir - get infrard data, independent of mask. However, only 
 *          the active infrared sensor readings are valid. It updates
 *          the State vector.
 */
int get_ir(void);

/*
 * get_sn - get sonar data, independent of mask. However, only 
 *          the active sonar sensor readings are valid. It updates
 *          the State vector.
 */
int get_sn(void);

/*
 * get_rc - get robot configuration data (x, y, th, tu), independent of 
 *          mask. It updates the State vector.
 */
int get_rc(void);

/*
 * get_rv - get robot velocities (translation, steering, and turret) data,
 *
 independent of mask. It updates the State vector.
 */
int get_rv(void);

/*
 * get_cp - get compass data, independent of mask. It updates the
 *          State vector.
 */
int get_cp(void);

/*
 * get_bp - get bumper data, independent of mask. It updates the State
 *          vector.
 */
int get_bp(void);

/*
 * get_ls - get laser data point mode, independent of mask. It updates
 *          the Laser vector.
 */
int get_ls(void);

/*
 * get_sg - get laser data line mode, independent of mask. It updates
 *          the laser vector.
 */
int get_sg(void);

/*
 * get_rpx - get the position of all nearby robots
 */
int get_rpx(long *robot_pos);

/***************************** 
 *                           *
 * World Interface Functions *
 *                           *
 *****************************/

/*
 * add_obstacle - creates an obstacle and adds it to the obstacle list
 *                of the robot environment. 
 * 
 * parameters:
 *    long obs[2*MAX_VERTICES+1] -- 
 *                The first element of obs specifies the number of 
 *                vertices of the polygonal obstacle (must be no greater 
 *                than MAX_VERTICES). The subsequent elements of obs 
 *                specifies the x and y coordinates of the vertices, 
 *                in counter-clockwise direction.
 */
int add_obstacle(long obs[2*MAX_VERTICES+1]);

/*
 * add_Obs - is the same as add_obstacle, for backward compatibility
 */
int add_Obs(long obs[2*MAX_VERTICES+1]);

/*
 * delete_obstacle - deletes an obstacle specified by obs from the robot 
 *                   environment 
 * parameters:
 *    long obs[2*MAX_VERTICES+1] -- 
 *                The first element of obs specifies the number of 
 *                vertices of the polygonal obstacle (must be no greater 
 *                than MAX_VERTICES). The subsequent elements of obs 
 *                specifies the x and y coordinates of the vertices, 
 *                in counter-clockwise direction.
 */
int delete_obstacle(long obs[2*MAX_VERTICES+1]);

/*
 * delete_Obs - is the same as delete_obstacle, for backward compatibility
 */
int delete_Obs(long obs[2*MAX_VERTICES+1]);

/*
 * move_obstacle - moves the obstacle obs by dx along x direction and 
 *                 dy along y direction. obs is modified.
 *
 * parameters:
 *    long obs[2*MAX_VERTICES+1] -- 
 *                The first element of obs specifies the number of 
 *                vertices of the polygonal obstacle (must be no greater 
 *                than MAX_VERTICES). The subsequent elements of obs 
 *                specifies the x and y coordinates of the vertices, 
 *                in counter-clockwise direction.
 *    long dx, dy -- the x and y distances to translate the obstacle
 */
int move_obstacle(long obs[2*MAX_VERTICES+1], long dx, long dy);

/*
 * move_Obs - is the same as move_obstacle, for backward compatibility
 */
int move_Obs(long obs[2*MAX_VERTICES+1], long dx, long dy);

/*
 * new_world - deletes all obstacles in the current robot world
 */
int new_world(void);


/*
 * place_robot - places the robot at configuration (x, y, th, tu). 
 *               In simulation mode, it will place both the Encoder-robot
 *               and the Actual-robot at this configuration. In real robot
 *               mode, it will call dp(x, y) and da(th, tu).
 * 
 * parameters:
 *    long x, y -- x-y position of the desired robot configuration
 *    int th, tu -- the steering and turret orientation of the robot
 *                  desired configuration
 */
int place_robot(long x, long y, int th, int tu);

/*******************************
 *                             *
 * Graphic Interface Functions *
 *                             *
 *******************************/

/*
 * draw_robot - this function allows the client to draw a robot at
 *              configuration x, y, th, tu (using the robot world 
 *              coordinates). 
 * 
 * parameters:
 *    long x, y -- the x-y position of the robot.
 *    int th, tu -- the steering and turret orientation of the robot
 *    int mode - the drawing mode. If mode = 1, the robot is drawn in 
 *              BlackPixel using GXxor (using GXxor you can erase the trace 
 *              of robotby drawing over it). If mode = 2, the robot is 
 *              drawn in BlackPixel using GXxor and in addition, a small arrow
 *              is drawn at the center of the robot using GXcopy (using this 
 *              mode you can leave a trace of small arrow). If mode = 3, 
 *              the robot is drawn in BlackPixel using GXcopy. When mode > 3,
 *              the robot is drawn in color using GXxor.
 */
int draw_robot(long x, long y, int th, int tu, int mode);

/*
 * draw_line - this function allows the client to draw a line from
 *             (x1, y1) to (x2, y2) (using the robot world coordinates). 
 *
 * parameters:
 *    long x1, y1, x2, y2 -- the two end-points of the line
 *    int mode -- the mode of drawing: when mode is 1, the drawing is 
 *                done in BlackPixel using GXcopy; when mode is 2, the drawing
 *                is done in BlackPixel using GXxor, when mode > 2, the drawing
 *                is done in color using GXxor.
 */
int draw_line(long x1, long y1, long x2, long y2, int mode);

/*
 * draw_arc - this function allows the client to draw arc which is part
 *            of an ellipse (using the robot world coordinates). 
 *
 * parameters:
 *    long x0, y0, w, h -- (x0, y0) specifies the upper left corner of the 
 *                          rectangle bounding the ellipse while w and h
 *                          specifies the width and height of the bounding 
 *                          rectangle, respectively.
 *    int th1, th2 -- th1 and th2 specifies the angular range of the arc.
 *    int mode -- the mode of drawing: when mode is 1, the drawing is 
 *                done in BlackPixel using GXcopy; when mode is 2, the drawing
 *                is done in BlackPixel using GXxor, when mode > 2, the drawing
 *                is done in color (GXxor)
 */
int draw_arc(long x0, long y0, long w, long h, int th1, int th2, int mode);

/*
 * get_robot_conf - interactively getting the robot's conf, by clicking
 *                  the mouse in the server's Robot window
 * 
 * parameters:
 *    long *conf -- should be an array of 4 long integers. The configuration
 *                  of the robot is returned in this array.
 */
int get_robot_conf(long *conf);

/*************************************
 *                                   *
 * Miscellaneous Interface Functions *
 *                                   *
 *************************************/
/*
 * server_is_running - this function queries the server to see
 *                     if it is up and running.  If so, this function
 *                     returns a TRUE, otherwise it returns FALSE.
 *                     This function is replaced by connect_robot, but 
 *                     is defined here for backward compatibility
 */
int server_is_running();

/*
 * quit_server - this function allows the client to quit the server
 *               assuming this feature is enabled in the setup file
 *               of the server
 */
int quit_server(void);

/*
 * real_robot - this function allows the client to switch to
 *              real robot mode in the server
 */
int real_robot(void);

/*
 * simulated_robot - this function allows the client to switch to
 *                   simulated robot mode in the server
 */
int simulated_robot(void);

/*
 * predict_sensors - this function predicts the sensor reading of
 *                   the robot assuming it is at position (x, y)
 *                   and orientation th and tu using the map of the
 *                   simulated robot environment. The predicted sensor
 *                   data values are stored in "state" and "laser".
 * 
 * parameters:
 *    long x, y, th, tu -- the configuration of the robot
 *    long *state -- where to put the predicted state data
 *    int *laser -- where to put the predicted laser data
 */
int predict_sensors(long x, long y, int th, int tu, long *state, int *laser);

/* 
 * this is something special for Greg at Yale
 */
int process_mcheck_reply(struct reply_struct *the_reply, double collide[3]);

/* 
 * this is something special for Greg at Yale
 */
int motion_check(long type, double a1, double a2, double a3, double a4,
		 double a5, double a6, double a7, double collide[3]);

/*
 * send_message
 */
int send_message(void);

/*
 * receive_message
 */
int receive_message(void);

/*
 * special_request - sends a special request (stored in user_send_buffer) 
 *                   to the robot and waits for the robot's response (which
 *                   will be stored in user_receive_buffer). 
 * 
 * parameters:
 *    char *user_send_buffer -- stores data to be sent to the robot
 *    char *user_receive_buffer -- stores data received from the robot
 */
int special_request(char *user_send_buffer, char *user_receive_buffer);

/*
 * init_sensors - initialize the sensor mask, Smask, and send it to the
 *                robot. It has no effect on the sensors 
 */
void init_sensors(void);

/*
 * init_mask - initialize the sensor mask, Smask.
 */
void init_mask(void);

/* 
 * mysleep - sleep for specified number of micro (10^-6) seconds
 *
 * parameters:
 *    long msec -- time to sleep in micro seconds
 */
int mysleep(long msec);

/*******************************************
 *                                         *
 * The following are helper functions for  *
 * developing user defined host <-> robot  *
 * communication                           *
 *                                         *
 *******************************************/

/*
 *  init_receive_buffer - sets the index to 4 which is the point
 *  at which data should begin to be extracted
 * 
 *  parameters:
 *     int *index -- is the buffer index
 */
int init_receive_buffer(int *index);

/*
 *  extract_receive_buffer_header - extracts the header information:
 *  length, serial_number, and packettype from the beginning of the
 *  receive buffer.
 *
 *  parameters:
 *     int *length -- is the returns the number of chars in the buffer
 *
 *     unsigned char *serial_number -- returns the serial number to be
 *                                     assigned to the packet
 *     unsigned char *packet_type -- returns the type number to be
 *                                   assigned to the packet
 *     unsigned char *buffer -- is the receive buffer
 */
int extract_receive_buffer_header(int *length, 
				  unsigned char *serial_number, 
				  unsigned char *packet_type, 
				  unsigned char *buffer);

/*
 *  init_send_buffer - sets the index to 4 which is the point
 *  at which data should be inserted
 *
 *  parameters:
 *     int *index -- is the buffer index
 */
int init_send_buffer(int *index);

/*
 *  stuff_send_buffer_header - loads the header information,
 *  length,serial_number, and packettype into the beginning of the
 *  buffer.  It should be called after the data has been stuffed,
 *  i.e. index represents the length of the packet.
 *
 *  parameters:
 *     int index -- is the buffer index which holds the number of chars
 *                  in the buffer
 *     unsigned char serial_number -- holds the serial number to be
 *                                    assigned to the packet
 *     unsigned char packet_type -- holds the type number to be
 *	                           assigned to the packet
 *
 *     unsigned char *buffer -- is the send buffer
 */
int stuff_send_buffer_header(short index, unsigned char serial_number, 
			     unsigned char packet_type, unsigned char *buffer);

/*
 *  stuffchar -  stuffs a 1 byte char into the send buffer
 *
 *  parameters:
 *     char data -- is the char to be stuffed
 *     unsigned char *buffer -- is the send buffer
 *     int *index -- is the buffer index which will be incremented to
 *                   reflect the bytes stuffed into the buffer
 */
int stuffchar(char data, unsigned char *buffer, int *index);

/*
 *  stuff2byteint - stuffs a short int(2 bytes) into the send buffer
 *
 *  parameters:
 *     int data -- is the value which will be split apart and stuffed
 *	           bytewise into the send buffer
 *     unsigned char *buffer -- is the send buffer
 *     int *index -- is the buffer index which will be incremented to
 *	             reflect the bytes stuffed into the buffer
 */
int stuff2byteint(short data, unsigned char *buffer, int *index);

/*
 *  stuff4byteint - stuffs a long int(4 bytes) into the send buffer
 *
 *  parameters:
 *     long data -- is the value which will be split apart and stuffed
 *	            bytewise into the send buffer
 *     unsigned char *buffer -- is the send buffer
 *     int *index -- is the buffer index which will be incremented to
 *	             reflect the bytes stuffed into the buffer
 */
int stuff4byteint(long data, unsigned char *buffer, int *index);

/*
 *  stuffuchar -  stuffs an unsigned char into the send buffer
 *
 *  parameters:
 *     unsigned char data -- is the char to be stuffed
 *     unsigned char *buffer -- is the send buffer
 *     int *index -- is the buffer index which will be incremented to
 *                   reflect the bytes stuffed into the buffer
 */
int stuffuchar(unsigned char data, unsigned char *buffer, int *index);

/*
 *  stuff2byteuint - stuffs an unsigned short int(2 bytes) into the send buffer
 *
 *  parameters:
 *     unsigned short data -- is the value which will be split apart and 
 *                            stuffed bytewise into the send buffer
 *     unsigned char *buffer -- is the send buffer
 *     int *index -- is the buffer index which will be incremented to
 *	             reflect the bytes stuffed into the buffer
 */
int stuff2byteuint(unsigned short data, unsigned char *buffer, int *index);

/*
 *  stuff4byteuint - stuffs an unsigned long int(4 bytes) into the send buffer
 *
 *  parameters:
 *     unsigned long data -- is the value which will be split apart and stuffed
 *	                     bytewise into the send buffer
 *     unsigned char *buffer -- is the send buffer
 *     int *index -- is the buffer index which will be incremented to
 *	             reflect the bytes stuffed into the buffer
 */
int stuff4byteuint(unsigned long data, unsigned char *buffer, int *index);

/*
 *  stuffdouble - stuffs a double(8 bytes) into the send buffer
 *
 *  parameters:
 *     double data -- is the value which will be split apart and stuffed
 *	              bytewise into the send buffer
 *     unsigned char *buffer -- is the send buffer
 *     int *index -- is the buffer index which will be incremented to
 *	             reflect the bytes stuffed into the buffer
 */
int stuffdouble(double data, unsigned char *buffer, int *index);

/*
 *  extractchar -  extracts a char from the receive buffer
 *
 *  parameters:
 *     unsigned char *buffer -- is the receive buffer which holds the data
 *     int *index -- is the receive buffer index which will be incremented to
 *                   reflect the position of the next piece of data to be 
 *         	     extracted
 */
char extractchar(unsigned char *buffer, int *index);

/*
 *  extract2byteint -  extracts a short int(2 bytes) from the receive buffer
 *
 *  parameters:
 *     unsigned char *buffer -- is the receive buffer which holds the data
 *     int *index -- is the receive buffer index which will be incremented to
 *                   reflect the position of the next piece of data to be 
 *         	     extracted
 */
short extract2byteint(unsigned char *buffer, int *index);

long extract4byteint(unsigned char *buffer, int *index);

/*
 *  extractuchar -  extracts an unsigned char from the receive buffer
 *
 *  parameters:
 *     unsigned char *buffer -- is the receive buffer which holds the data
 *     int *index -- is the receive buffer index which will be incremented to
 *                   reflect the position of the next piece of data to be 
 *         	     extracted
 */
unsigned char extractuchar(unsigned char *buffer, int *index);

/*
 *  extract2byteuint -  extracts an unsigned short int(2 bytes) from the 
 *                      receive buffer
 *
 *  parameters:
 *     unsigned char *buffer -- is the receive buffer which holds the data
 *     int *index -- is the receive buffer index which will be incremented to
 *                   reflect the position of the next piece of data to be 
 *         	     extracted
 */
unsigned short extract2byteuint(unsigned char *buffer, int *index);

/*
 *  extract4byteuint -  extracts an unsigned long int(4 bytes) from the 
 *                      receive buffer
 *
 *  parameters:
 *     unsigned char *buffer -- is the receive buffer which holds the data
 *     int *index -- is the receive buffer index which will be incremented to
 *                   reflect the position of the next piece of data to be 
 *         	     extracted
 */
unsigned long extract4byteuint(unsigned char *buffer, int *index);

/*
 *  extractdouble -  extracts a double(8 bytes) from the receive buffer
 *
 *  parameters:
 *     unsigned char *buffer -- is the receive buffer which holds the data
 *     int *index -- is the receive buffer index which will be incremented to
 *                   reflect the position of the next piece of data to be 
 *         	     extracted
 */
double extractdouble(unsigned char *buffer, int *index);

/************************************************
 *                                              *
 * Global variable access functions for Allegro * 
 * Common Lisp interface                        *
 *                                              *
 ************************************************/

int get_state(long state[NUM_STATE]);

int get_laser(int laser[2*NUM_LASER+1]);

int get_mask(int mask[NUM_MASK]);

int set_mask(int mask[NUM_MASK]);

int set_server_machine_name(char *sname);

int set_serv_tcp_port(int port);
