#ifndef ASPECT_H
#define ASPECT_H


/* File used by Aspect.c ONLY */

/*************************************************
 *                                               *
 * Nomadic Host Software Development Environment *
 *                                               *
 * The worldmodel.h file                         *
 *   This is the include file that contains the  *
 *   world model.                                *
 *                                               *
 * First Created: 10/18/92                       *
 * Last Modified: 03/08/93                       *
 *                                               *
 * Author:        David J. Zhu                   *
 *                                               *
 * Copyright 1992, Nomadic Technologies, Inc.    *
 *                                               *
 *************************************************/

/*
 */
#define PI           3.1415926
#define HALF_PI      1.5707963
#define TWO_PI       6.2831852
#define PI_HALF      4.7123890

#define DEG2RAD      0.0174533
#define RAD2DEG      57.295828

#define R_RADIUS     90  /* radius of robot in 0.1 inch unit */

#define TRUE         1
#define FALSE        0

#define CLOCKS_PER_SEC 1000000

/*
 * constants world model
 */
#define MAX_VERTEX_MEM  3000
#define MAX_LINE_MEM    3000
#define MAX_PLINE_MEM   3000
#define MAX_CURVE_MEM   3000
#define MAX_MAP_MEM     3000
    
#define LINE_SEGMENT    0
#define ARC             1

#define MAX_VERTICES    10    /* maximum number of verices in a poly */


#define MAX_ROBOTS 10

#define MAX_MESSAGE 100

#define NUM_LASER 482

/* 
 * World Modeling 
 */
struct rect_type {
    double x1;
    double y1;
    double x2;
    double y2;
    struct rect_type *next_rect;
};

struct vertex_type {
    double x;
    double y;
    double n; 
    struct vertex_type *next_vert;
};

struct vertex_memory_type {
    struct vertex_type *vert_list[MAX_VERTEX_MEM];
    long length, begin_ptr, end_ptr;
};

struct line_type {
    long id; 
    double x1, y1, x2, y2;
    double n;        /* n is the outward normal */
    struct line_type *next_line;
};

struct line_memory_type {
    struct line_type *line_list[MAX_LINE_MEM];
    long length, begin_ptr, end_ptr;
};

typedef struct polar_line_segment_type { /* this polar line equation is derived
				    from the two end points of the edge
				    of the polygon wrpt the center of the 
				    robot */
    long id;
    double x01, y01, x12, y12;    /* equation parameters */
    double t1, t2, r1, r2;        /* bounds */
    double n;                     /* outward normal */
    struct polar_line_segment_type *next_pline;
}PolLine;

struct pline_memory_type {
    struct polar_line_segment_type *pline_list[MAX_PLINE_MEM];
    long length, begin_ptr, end_ptr;
};

struct line_segment_type {
   struct rect_type *rb;      /* bounding box of the line segment */
   struct vertex_type *v,*vp; /* first and second points on the line segment */
   double n;                  /* outward normal of the line segment */
};

struct arc_of_circle_type {
    struct rect_type *rb;
    struct vertex_type *c; /* center of circle */
    double th1, th2, r;    /* radius of the arc */
    long dir;               /* direction of the arc, 1 if counter clockwise */
};

struct curve_type {
    long type; /* specifies whether it is a line segment of an arc */
    struct line_segment_type *linesegment;
    struct arc_of_circle_type *arc;
    struct curve_type *next_curve;
};

struct curve_memory_type {
    struct curve_type *curve_list[MAX_CURVE_MEM];
    long length, begin_ptr, end_ptr;
};

struct poly_type {
    struct rect_type *rect_bound;
    struct vertex_type *vert_list;
    long vlist[2*MAX_VERTICES+1];
    struct line_type *line_list;
};

struct map_type {
    struct poly_type        *poly;
    struct map_type         *next_poly;
};

struct map_memory_type {
    struct map_type *map_list[MAX_MAP_MEM];
    long length, begin_ptr, end_ptr;
};

typedef enum ANGLE_TYPE {
	ANGLE_SHORT_SEG,
	ANGLE_LEFT,
	ANGLE_RIGHT
} ANGLE_TYPE;

typedef struct angle_list { 
  double th;
  PolLine *pline;
  ANGLE_TYPE type;
  struct angle_list *next;
}AList;

typedef struct short_polLine {
	double t1, t2;
	PolLine *pline;
	struct short_polLine *next;
} short_polLine;

typedef struct sglptr_list { 
  Sgl *sglptr;
  struct sglptr_list *next;
}sglptr_list;

#endif









