#ifdef __cplusplus 
extern "C"  { 
#endif 
/************************************************************************
  This file maintains a list of possible trasformations
  PossibleTransf of structures "Hyp" defined in 
  localize.h 

  each struture has a field called Tr[3] which defines the transform
  from the robot encoder position and the absolute location 
  corresponding to the hypothesis.

  calling rob_enc[4] = (x,y,sterring,turret), the root encoder position
          abs_loc[4] = (",",  " , "  "     ), the absolute location
          
     we have

     abs_loc[0] = cos(Tr[2])*rob_enc[0] - sin(Tr[2])*rob_enc[1] + Tr[0]
     abs_loc[1] = sin(Tr[2])*rob_enc[0] + cos(Tr[2])*rob_enc[1] + Tr[1]
     abs_loc[2] = rob_enc[2] + Tr[2];
     abs_loc[3] = rob_enc[3] + Tr[2]

  At the end of the localization process, each hypothesis defines an absolute
  location abs_loc(hyp).

  -The function TransformListHyp(list_hypo,rob_enc)  
  transforms the absolute locations of every hypothesis 
  in list_hypo into  the transformation Tr. 

  rob_enc and vrob_enc is the encoder location of the robot at the time 
  the localization was made.

  
  After the robot will have moved, the absolute location corresponding 
  to each hypothesis will then be computed as above.
  
  
  Then, the process goes like this:

  -getting the new robot location and its variance after it moved
  -getting laser data at this location

  FilterBadHypo:

  -for each hypothesis, 
       -computes the corresponding absolute location
       -checks if this location is within the boudary limits 
       -check if the local data is valid for this location,
        (i.e, check if there is any data seen behind a static object).

  The  list of remaining hypothesis is stored in 
  the static local variable "PossibleTransf".

  
      
  Note about the way the structure Hyp is used in the list PossibleTransfs:
  the field ->lsm is no longer a list of matchings but a reference to
  the corresponding hypothesis in the original list ListHypo. 

**********************************************************************/

/**
  05/10/94 add the function GetListMatch() 
  returning the list (in PDBL format) of matchings (see localize.h 
  structure Smatch*) of the best current hypothesis
**/







extern double VarLoc[6];

/****************
  List of possible transformations "encoder position -> absolute position"
****************/


/*************
  Map structure and an auxilliary for 
  its local representation (in robot coordinate system)
*************/

/**************
  Structure for the boundary.
  A location will be rejected if the robot doens't
  belong to the interior of this boundary
***************/

/************
  for memorizing the best hypothesis
  computed by the Localization process
*************/

/* prototype of added functions (by David Zhu) */
long DeleteHypo(long Trx, long Try, long Trth, long *possloc);;




/**********************************************************************
-The function TransformListHyp()  
  transforms the absolute locations of every hypothesis 
  in list_hypo into a transformation "encoder position -> absolute position"
  See header of this file.

  lhyp = list of hypothesis to be transformed
  map = map !
  local_map = empty (but allocated !) struture for containing a local 
              representation of the map.
  bound = structure containing the boundary (can be NULL).
  xrob = encoder position of the robot when the localization was made.
  possloc = array of all possible locations.
  WARNING: possloc has to be allocated for 500 hypothesis = 2001 long integers.
  If there is more than 500 hypothesis, the remaining ones are not copied.

  Returns the index of the best location in the array possloc.
***********************************************************************/

long TransformListHyp(PDBL *lhyp,igla *map, igla *local_map,igla *bound,
		      double *xrob,long *possloc, int maxarraysize);


PDBL *GetListMatch(void);

/* Mainly for debug purpose */
Hyp *GetBestHyp(void);

/* Mainly for debug purpose */
PDBL *GetPossibleTranfs();

/****************
  Filters bad hypothesis by:
  

 Returns -1 if no possibilities
***************/




long FilterBadLocations(long n_data,long *possloc, double distmax, int maxarraysize);


/**************
delete all hypothesis corresponding to Trx Try and Trth
**************/


long DeleteHypo(long Trx, long Try, long Trth, long *possloc);


/** returns x,y and theta in meters and degrees **/
long BestCurrentLocation(double *x,double *y,double *theta);




/************
  compute the length of the total overlaping
  for the hypothesis hyp 
************/


void Compute_length_matched(Hyp *hyp);




long LasSegmentation(long nbp,long *laser_val);
   




















#ifdef __cplusplus 
 } 
#endif
