
/******* computes in sg2 the projection of sg1 through 
  the position xp and its variance vxp; 
  the variance of sg2 is computed according to 
  the variance of sg1 and vxp
***********/

void project_sg_unc(Sgl *sg1,Sgl *sg2,double *xp,double *vxp);
 
  
/**********
  computes the fusion between two segment s1 and s2;
  puts the result in s3;
  Conventions
    -: s1 and s2 are expressed in the same coordinate system (for ex. absolute).
***********/
  
void abs_segments_fusion(Sgl *s1, Sgl *s2, Sgl *s3);
  
  




/******
  Localizing functions
******/

/*****
  distance between s1 and the projection of s2 through
  the transformation pr
  s1 and s2 are given by (d,theta)
  pr = (x,y,phi)
*****/

void sgdist(double *s1,double *pr,double *s2,double *res);

void dsgdists(double *s1,double *pr,double *s2,double *res);
   


void dsgdistpr(double *s1,double *pr,double *s2,double *res);
   


/*****
  distance between s1 and the projection of s2 through
  the transformation pr
  s1 and s2 are given by (d,theta)
  pr = (x,y,phi)

Same thing as above, but s2 is encoded in s1 = (s1[2],s1[3])
*****/

void sg2dist(double *s1,double *pr,double *res);
    

void dsg2dists(double *s1,double *pr,double *res);



void dsg2distpr(double *s1,double *pr,double *res);
  







/**************

  xp, vp : estimate and variance on robot position.
  vpt =  variance on robot position for the test 
         of compatibility.
  ig : image perceived at this position
  s1 : segment belonging to ig in local coordinates
  s2 : segment belonging to the global  map.
  thr: threshold for numerical compatibility
  op: if not equal to 0, this is only a test of
      compatibility (numerical and geometrical)
      if equal to 0, proceeds also to the fusion
      process updating the robot position and its
      variance.


   Note: normally vpt and vp are idientical;
         however, one might want to use a different value
         for the test (i.e a constant matrix) and for the fusion.

  Side effect: modifies xp and vp (not vpt);

  Tests first if s1 and s2 are numerically compatible
  by comparing proba. distance with a threshold thr.
  If not, returns 0.

  Tests first the overlaping of s1 and s2 ;
  if not returns 2
  

  if the two previous tests are ok
      if op equals to 0, proceeds to the fusion process
  returns 1.
**************/



int sg_test_match_fus(igla *ig,double *xp,double *vp,
		      double * vpt,Sgl *s1,Sgl *s2,double thr,long op);
     

  

/*************
  same as above but s2 is also unaccurate
*************/

int sg_test_match_fus2(igla *ig,double *xp,double *vp,double *vpt,
		       Sgl *s1,Sgl *s2,double thr,int op);
     


/************
computes the robot position and its variance  (xr,vr) from
sg1 matched with s1 et sg2 matched with s2.
s1 and s2 are assumed totally accurate.
sg1 and sg2 are unaccurate with a variance ->vr,->vo,->vro .

The angle (x[2]) will be computed in the reference frame in which the
laser segments are expressed : in the turret reference frame or in
the steering.

 If the turret frame is used, don't use &Glob_pos directly since
Glob_pos[2] is the steering angle. 
********/

int Localize(Sgl *sg1,Sgl *s1,Sgl *sg2,Sgl *s2,double *xr,double *vr);
 


/************
Same thing as above but  s1 and s2 are also inaccurate

computes the robot position and its variance  (xr,vr) from
sg1 matched with s1 et sg2 matched with s2.
s1 and s2 are inaccurate 
sg1 and sg2 are inaccurate with a variance ->vr,->vo,->vro .

The angle (x[2]) will be computed in the reference frame in which the
laser segments are expressed : in the turret reference frame or in
the steering.

 If the turret frame is used, don't use &Glob_pos directly since
Glob_pos[2] is the steering angle. 
********/

int Localize2(Sgl *sg1,Sgl *s1,Sgl *sg2,Sgl *s2,double *xr,double *vr);



long Localize_np(Sgl *sg1,Sgl *s1,Sgl *sg2,Sgl *s2,double *xr);

 

/************
Updating the robot position from xp,vp and the matching
between s1 (uncertain)  and s2 (from the map);

modifies xp and vp.
************/

void Update_pos(double *xp,double *vp,Sgl *s1,Sgl *s2);
  



/************
Same thing as above but s2 is unacurate

Updating the robot position from xp,vp and the matching
between s1 (uncertain)  and s2 (from the map);

modifies xp and vp.
************/

void Update2_pos(double *xp,double *vp,Sgl *s1,Sgl *s2);
 

/*
 *  Uncertainty on Segments 
 */




/**********
  Computes the probabilistic distance
  between the projection of s1 trough
  the robot position (xp,vp) and s2.

  = test of compatibility of the local
  segment s1 and the global segment s2
  according to the current robot position
**********/

double sg_test_match(double *xp,double *vp,Sgl *s1,Sgl *s2);





/****
  added 5/20/94:

  returns the probabilistic distance betweenn s_data and smap 
  when s_data is transformed through the uncertan relationship 
  xp, vp;
  if op = 1
     Test also the overlapping s_map,s_data.
     Returns -1 if there is no overlapping.
     Note: one should do a test on the orientation before this test since 
     testing overlapping is meaningless if s_map and projection(s_data)
     are not parallel.
*****/

double sg_tr_sg_dist(igla *ig,double *xp,double *vpt,
		       Sgl *s_map,Sgl *s_data, long op);




/*********
  initialize a line with its variance from 
  p1 and p2 and a coefficient unc.

 

  Convention: ->o is the angle (radian) of the opposite normal 
              ->d is the orthogonal distance to the origin
	      ->vx = -cos(o)
	      ->vy = -sin(o)
	      vx,vy, is the normal going to the right when you walk from 
	      p1 to p2

 
  The model is :
  unc is the uncertainty in x and y for a segment supposed to be 100 inch
  long.
  which leads to an uncertainty of 1/10000 * unc^2/12  in angle.

  The uncertainty in d (distance in polar coordinates) is computed from this.

***********/
     

long initsg_map (p2d *p1,p2d *p2,Sgl *dt,double unc);
















void project_coord_sys_unc(double *p1,double *vp1, double *p2, double *vp2, double *f12, double *vf12);









