


/*****************
   test if one extremity of s1 is included in s2
 s1 is projected trough the transform in ig 
 returns 1 if yes, 0 if no.
 Uses the function Sg_pos in las_struct.c.
****************/


/***************
 * FUNCTION:     Test_overlap
 * PURPOSE:      Projects s1 through the transform 
                 defined in *ig (see Sg_pos() in las_struct.c)
		 calling pt1 and pt2 the two extremities of
		 the projected segment, this function tests
		 if the orthogonal projection of either pt1 or
		 pt2 on s2 line belongs to the segment s2.
		 In case the projection of s1 is parallel to
		 s2, this is equivalent to test the overlaping
		 of the projected s1 and s2.
		 returns 1 if overlaping, 0 otherwise.
		 Mainly used for localization and map building.
 * ARGUMENTS:    igla *ig :
 *               Sgl *s1 :
 *               Sgl *s2 :
 * ALGORITHM:    
 * RETURN:       int 
 * SIDE EFFECT:  
 * CALLS:    Sg_pos/las_struct.c    
 * CALLED BY:    
 ***************/

int Test_overlap(igla *ig,Sgl *s1,Sgl *s2);
double lengthOverlap(igla *ig,Sgl *s1,Sgl *s2, double *lengthUnderlap);
double angleOverlap(igla *ig,Sgl *s1,Sgl *s2, double *angleUnderlap);
     


/*****************

  v being the direction of s1,

  this function computes an array of 4 integers:
  
  t[0] = (s2->first - s1->first).v
  t[1] = (s2->first - s1->last).v


  t[2] = (s2->last - s1->first).v
  t[3] = (s2->last - s1->last).v

  
****************/

void overlap_abs(Sgl *s1,Sgl *s2,double *t);
     



/*************
  computes a segment image in local coordinates igloc 
  xp from a segment image in absolute coordinate igabs;

  only ->first,->last, ->vx,->vy and ->d  of the local 
  version are computed

  igloc is assumed allocated and initialized
  the list of segments in igloc is already 
  constructed and has to be of the same length 
  as the one of igabs
*************/


void tr_abs_to_loc(igla *igabs,igla *igloc,double *xp,char type);

/*******
  this function tests if sloc is behind smap with a 
  gap more or equal than thr;
  the distance is understood as the orthogonal projection 
  of the farthest point of the part of sloc possibly 
  occulted by smap.

  sloc and smap are assumed expressed in the local coord. sys.
  from where sloc has been seen.

*******/

int test_behind(Sgl *sloc,Sgl *smap, double thr);


/***********
  returns the maximum overlap 
***********/

double MaxOverlap(igla *ig,Sgl *s1,Sgl *s2);
     




/***********
  returns 1 if the polygone is counterclockwise
  -1 otherwise
  particular case: less than 3 segments returns 0
***********/

long poly_orientation(igla *pl);












