/*************************************************
 *                                               *
 * Nomadic Host Software Development Environment *
 *                                               *
 * The serv.h file                               *
 *   this is the include file for the IPC_server *
 *   and the IPC_client (they share structures)  *
 *                                               *
 * First Created: 04/05/92                       *
 * Last Modified: 08/29/92                       *
 *                                               *
 * Author:        David J. Zhu                   *
 *                                               *
 * Copyright 1992, Nomadic Technologies, Inc.    *
 *                                               *
 *************************************************/

/* $Log: serv.h,v $
/* Revision 1.1  2005/06/23 08:44:25  sjoyeux
/* Initial revision
/*
/* Revision 1.1.1.1  1999/02/18 11:23:17  matthieu
/* initial import
/*
 * Revision 1.1.1.2  1998/05/26 13:45:42  philippe
 * adding sick support : note this is not a real adding but just records a different version with sick support and no sensus 500
 *
 * Revision 1.1.1.1  1996/05/21 09:30:54  philippe
 * portage premiere version qui marche au laas, apres revision sur PC. A arranger : reprojection apres localisation, SpeedLevels
 *
 * Revision 1.1.1.1  1996/05/06 16:52:17  philippe
 * copie de la localisation modifie pour fonctionner au laas.
 * Main.c a ete perdu : a reecrire
 *
 * Revision 1.1.1.1  1996/04/17 11:59:06  philippe
 * Import initial de localisation.
 *
 * Revision 1.1.1.1  1995/09/07  19:13:19  root
 *
 * Revision 1.1.1.1  1995/07/11  21:42:50  philippe
 * Imported sources for the Map Building and Localization toolkit
 * Imported on July 11th 1995
 *
 * Revision 1.3  1993/07/21  22:24:30  kamason
 * Fixed LOG for header files
 *
 * Revision 1.2  1993/07/21  22:17:27  kamason
 * Added rcs LOG comment
 */

#include <stdio.h>              /* standard i/o functions*/
#include <errno.h>
#include <sys/types.h>          /* for "bind" system calls */
#include <sys/time.h>           /* for "select" system call */
#include <sys/socket.h>         /* for socket calls */
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <fcntl.h>
#include <signal.h>
#include <unistd.h>

#define MAX_MESSAGE             100

#define FALSE			0     /* general defines */
#define TRUE			1
#define NEW_SOCKET       	2

#define MAX_VERTICES            10    /* maximum number of verices in a poly */
#define MAX_HOST_NAME_LENGTH	100
#define MAX_MESG_LENGTH	        1200


#define IS_SERVER_ALIVE_MSG	0     /* message types */
#define IS_SERVER_ALIVE_ACK	1

/*
 *  request message types 
 */
#define AC_MSG                  10
#define SP_MSG                  11
#define VM_MSG                  12
#define PR_MSG                  13
#define PA_MSG                  14
#define GS_MSG                  15
#define CT_MSG                  16
#define ST_MSG                  17
#define LP_MSG                  18
#define TK_MSG                  19
#define ZR_MSG                  20
#define DP_MSG                  21
#define CONF_IR_MSG             22
#define CONF_SN_MSG             23
#define CONF_CP_MSG             24
#define CONF_LS_MSG             25
#define CONF_TM_MSG             26
#define GET_IR_MSG              27
#define GET_SN_MSG              28
#define GET_RC_MSG              29
#define GET_RV_MSG              30
#define GET_RA_MSG              31
#define GET_CP_MSG              32
#define GET_LS_MSG              33
#define SETUP_LS_MSG            34
#define GET_BP_MSG              35
#define CONF_SG_MSG             36
#define GET_SG_MSG              37
#define DA_MSG                  38
#define WS_MSG                  39

#define RMOVE_MSG               40
#define RPLACE_MSG              41
#define PREDICTSENSOR_MSG       42

#define ADDOBS_MSG  	        50
#define DELETEOBS_MSG           51
#define MOVEOBS_MSG             52
#define NEWWORLD_MSG            53

#define DRAWROBOT_MSG           60
#define DRAWLINE_MSG            61
#define DRAWARC_MSG             62

#define QUIT_MSG                70
#define REALROBOT_MSG           71
#define SEND_MESSAGE_MSG        72
#define RECEIVE_MESSAGE_MSG     73
#define SIMULATEDROBOT_MSG      74

/*
 * reply message types 
 */

#define STATE_MSG               80
#define MOVED_MSG               81
#define REPLY_MSG               82 
#define INFRARED_MSG            83
#define SONAR_MSG               84
#define LASER_MSG               85
#define COMPASS_MSG             86
#define BUMPER_MSG              87
#define CONFIGURATION_MSG       88
#define VELOCITY_MSG            89
#define ACCELERATION_MSG        90
#define ERROR_MSG               91

#define CREATE_ROBOT_MSG        101
#define CONNECT_ROBOT_MSG       102
#define DISCONNECT_MSG          103

#define GET_CONF_MSG            200

#define SPECIAL_MSG             300

#define MAX_VERT     10
#define NUM_STATE    45
#define NUM_MASK     44 
#define NUM_LASER    482 
#define MAX_MESSAGE  100
#define MAX_USER_BUF 1024

#define SERIAL_ERROR 11
#define IPC_ERROR    111

/* these type definitions are for user define package 
   processing */
typedef union
	{
	char            bytes[8];
	double          data;
	}double_union;

typedef union
	{
	char	        bytes[4];
	short		words[2];
	long	        data;
	}long_union;

typedef union
	{
	unsigned char	bytes[2];
	short		data;
	}short_union;

typedef union
	{
	unsigned char   bytes[2];
	unsigned short  data;
	}ushort_union;

typedef union
	{
	unsigned char	bytes[4];
	unsigned short	words[2];
	unsigned long	data;
	}ulong_union;

struct request_struct
{
    short type;
    short size;
    long  mesg[MAX_MESG_LENGTH];
};

struct reply_struct
{
    short type;
    short size;
    long  mesg[MAX_MESG_LENGTH];
};

